#!/bin/awk -f

BEGIN {
	point = "/"
	max = 1

# get spool directiry
	cmd = "/usr/sbin/postconf -h mail_spool_directory"
	cmd | getline SPOOL
	close(cmd)

# check if it is a link
	cmd = "/bin/readlink -f " SPOOL
	cmd | getline SPOOL
	close(cmd)

# -r <file> means read spool mount point
# -w <file> means write new fstab with usrquota enabled
# otherwise read <file> or <stdin> and show new fstab
	if (ARGC > 2) {
		if (ARGV[1] == "-r")
			mode = "read"
		if (ARGV[1] == "-w")
			mode = "write"
		ARGV[1] = ARGV[2]
		delete ARGV[2]
		ARGC--
	}
}

# skip empy lines and comments
/^(#|[[:space:]]*$)/ {
	if (orig_fstab)
		orig_fstab = orig_fstab "\n" $0
	else
		orig_fstab = $0
	if (new_fstab)
		new_fstab = new_fstab "\n" $0
	else
		new_fstab = $0
	next
}

{
	# check if mountpoint is quoted
	if (match($0, /^([^[:space:]]+[[:space:]]+)'([^']+)'([[:space:]]+[^[:space:]]+)([[:space:]]+[^[:space:]]+)([[:space:]]+[^[:space:]]+)?([[:space:]]+[^[:space:]]+)?$/, list) > 0) {
		fs_file = list[2]
		fs_mntops = list[4]
	}
	else {
		fs_file = $2
		fs_mntops = $4
	}

	# is new mountpoint more appropriate?
	regex = "^" fs_file
	if (SPOOL ~ regex && max <= length(fs_file)) {
		max = length(fs_file)
		point = fs_file

		# add usrquota if it hasn't one
		if (fs_mntops !~ /usrquota/)
			line = gensub(fs_mntops, "usrquota", "g")
		else line = $0
		# remove "defaults" keyword
		sub("defaults,", "", line)
		new_fstab = orig_fstab "\n" line
	}
	else
		new_fstab = new_fstab "\n" $0
	orig_fstab = orig_fstab "\n" $0
}

END {
	switch(mode) {
		case "read":
			# print mountpoint
			print point
			break
		case "write":
			# write new fstab
			print new_fstab > ARGV[1] 
			close(ARGV[1])
			break
		default:
			# show new fstab
			print new_fstab
	}
}
